#!/bin/bash

display_help() {
	echo "Usage: ./cam_regs.sh -r <reg.txt> [-s <subdev>] [-d <videodev>]"
}

OPTIONS='hs:r:d:'
S='subdev14'
D='/dev/video0'
REG=false

while getopts $OPTIONS option
do
	case $option in
		d ) D=$OPTARG;;
		r ) REG=$OPTARG;;
		s ) S=$OPTARG;;
		h  ) display_help; exit;;
		\? ) echo "Unknown option: -$OPTARG" >&2; exit 1;;
		:  ) echo "Missing option argument for -$OPTARG" >&2; exit 1;;
		*  ) echo "Unimplemented option: -$OPTARG" >&2; exit 1;;
	esac
done

if [ "$REG" = "false" ]; then
	echo "No register-settings.txt supplied"
	display_help
	exit 1
fi

CLEAN=$(sed s/\#.*$// ${REG} | sed '/^\s*$/d' | tr -d '[:blank:]')

while IFS=',' read -r ADDR REG VALUE OPT; do
	#echo "$ADDR $REG $VALUE $OPT"
	if [[ $OPT == "" ]]; then
		#perl -e 'printf "reg 0x%X new value = 0x%X (%016b b)\n\n",'$REG','$VALUE','$VALUE
		v4l2-dbg -c ${S} -d ${D} -s ${REG} ${VALUE} -w2
	else
		INPUT=$(v4l2-dbg -c ${S} -d ${D} -g ${REG})
		READ_VALUE=$(echo $INPUT | tr "\n" " " | cut -d" " -f5 | tr "h" " ")
		READ_VALUE=$(printf 0x%s $READ_VALUE)
		if [[ $OPT == "a" ]]; then
			VALUE=$((VALUE & READ_VALUE))
		elif [[ $OPT == "o" ]]; then
			VALUE=$((VALUE | READ_VALUE))
		else
			echo "Unidentified option found!"
		fi

		#perl -e 'printf "reg 0x%X new value = 0x%X (%016b b)\n\n",'$REG','$VALUE','$VALUE
		v4l2-dbg -c ${S} -d ${D} -s ${REG} ${VALUE} -w2
	fi
done <<< $CLEAN
